/*  
 	HEX2ENT

	Utility to convert Intel Hex to .ENT files for the Sol-20. Only
	16-bit hex files and a 64K address space are supported (no segment
  	records). The output is written with only CR at the end of lines
	instead of CR/LF. This allows the Sol-20 to keep up at 9600 baud
	with a 100ms delay at each CR from the terminal emulator.

	Usage: HEX2ENT fname[.ext] (.hex default)
	Where: .ext = .hex or .ent

		If ext <> "ent", then the input file is assumed to be
		Intel hex and is converted to an ent file. 

		If ext = "ent" then the input ent file is written to
		fname-NOLF.ent with only a CR at the end of each line
		(not CR/LF).

	Written by Mike Douglas
	
	 Ver     Date             Desc
	----   ----------    ------------------------------------------------ 
	 1.0   03/02/2024	Initial version
	 1.1   03/06/2024	Add ability to strip line feeds from an
				input .ent file
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define FILENAME_LEN 81
#define INBUF_LEN 300

/* display_usage - Display how the program is used and then exit */

void display_usage (void)
{
	fprintf(stderr, "\nUsage: HEX2ENT fname[.ext]\n\n"
					    "   If ext <> 'ent', then the input file is assumed to be Intel hex and\n"
						 "   is converted to an ent file. The output file is fname.ent\n\n"
						 "   If ext = 'ent' then the input ent file is written to fname-NOLF.ent\n"
						 "   with only a CR at the end of each line (not CR/LF).\n\n");
	exit(1);
}

/* main */

int main (int argc, char *argv[])
{
	char	fileName[FILENAME_LEN];
	char	inBuf[INBUF_LEN];
	char	dataStr[INBUF_LEN];

	FILE	*inFile, *outFile;
	int	byteCount, dataAddr, recType, i;
	char	*bufPtr;
	int	haveStartAddr = 0;
	int	inputIsEnt = 0;
	
/* Process command line parameters */

	if ((argc < 2) || (argc > 2))
		display_usage();

	strcpy(fileName, argv[1]);
	if ((bufPtr = strrchr(fileName,'.')) == NULL)
		strcat(argv[1],".hex");						/* add default extension */
	else {
		*bufPtr = '\0';   							/* remove ext from out file name */
		if ((0==strcmp(bufPtr+1,"ent")) || (0==strcmp(bufPtr+1,"ENT"))) {
			inputIsEnt = 1;
			strcat(fileName,"-NOLF");
		}
	}

	if (islower(fileName[0]))
		strcat(fileName, ".ent");					/* output file */
	else
		strcat(fileName, ".ENT");					/* output file */

	if ((inFile = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr,"Can't open file %s\n", argv[1]);
		exit(1);
	}

	if ((outFile = fopen(fileName, "w")) == NULL) {
		fprintf(stderr,"Can't open file %s\n", fileName);
		exit(1);
	}

/* Read the hex file and output .ENT lines */

	do {
		if (NULL==fgets(inBuf, INBUF_LEN, inFile))
			continue;
		i = strlen(inBuf) - 1;
		if (i > (INBUF_LEN - 10)) {
			fprintf(stderr,"\nInput 'ent' file already has LF's stripped\n");
			exit(1);
		}
		if (inBuf[i] == '\n')
			inBuf[i] = '\0';

/* .ent to -NOLF .ent */

		if (inputIsEnt) {
			fprintf(outFile,"%s\r",inBuf);
			continue;
		}

/* .hex to .ent conversion */

		sscanf(inBuf, ":%2x%4x%2x%s", &byteCount, &dataAddr, &recType, dataStr);
		bufPtr = dataStr;

		if ((recType == 0) && (byteCount > 0) ) {
			if (!haveStartAddr) {
				haveStartAddr = 1;
				fprintf(outFile, "ENT %04X", dataAddr);		/* initial ENT command */
			}
			fprintf(outFile, "\r%04X:", dataAddr);				/* supplemental line address */
			do {
				i = (byteCount > 16) ? 16 : byteCount;			/* handle lines over 16 bytes */
				byteCount -= i;
				do {
					fprintf(outFile, " %C%C", bufPtr[0], bufPtr[1]);
					bufPtr += 2;
				}
				while (--i != 0);
				if (byteCount > 0)									/* going to a second line, need */
					putc('\r', outFile);								/* CR for the emulator to see */
			} while (byteCount > 0);
		} 
	} while (!feof(inFile));

	if (!inputIsEnt) 
		fprintf(outFile,"/\r");	
	fclose(inFile);
	fclose(outFile);
	fprintf(stderr,"\n'%s' converted to '%s'\n", argv[1], fileName);
	return 0;
}
